"""
aiopyfcm
"""

from .message import Message
from .authenticator import PyFCMAuthenticator
from .aiopyfcm import AioPyFCM
from .errors import (
    InvalidCredentialsError,
    AioPyFCMError,
    UnspecifiedError,
    InvalidArgumentError,
    UnregisteredError,
    SenderIdMismatchError,
    QuotaExceededError,
    UnavailableError,
    InternalServerError,
    ThirdPartyAuthError
)


__TITLE__ = 'aiopyfcm'
__SUMMARY__ = 'aiopyfcm is Python Asynchronous FCM Push Controller'
__URL__ = 'https://github.com/837477/aiopyfcm'
__VERSION__ = "0.2.1"
__AUTHOR__ = '837477'
__EMAIL__ = '8374770@gmail.com'
__LICENSE__ = 'MIT License'


__all__ = [
    'Message',
    'PyFCMAuthenticator',
    'AioPyFCM',
    'InvalidCredentialsError',
    'AioPyFCMError',
    'UnspecifiedError',
    'InvalidArgumentError',
    'UnregisteredError',
    'SenderIdMismatchError',
    'QuotaExceededError',
    'UnavailableError',
    'InternalServerError',
    'ThirdPartyAuthError',
    __TITLE__,
    __SUMMARY__,
    __URL__,
    __VERSION__,
    __AUTHOR__,
    __EMAIL__,
    __LICENSE__
]



"""
aiopyfcm
"""
from setuptools import setup
from aiopyfcm import (
    __TITLE__, __SUMMARY__, __URL__, __VERSION__, __AUTHOR__, __EMAIL__, __LICENSE__
)

with open("README.md", "r", encoding='utf-8') as f:
    long_description = f.read()

setup(
    name=__TITLE__,
    version=__VERSION__,
    description=__SUMMARY__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=__URL__,
    author=__AUTHOR__,
    author_email=__EMAIL__,
    license=__LICENSE__,
    packages=['aiopyfcm'],
    keywords=__SUMMARY__,
    install_requires=['requests', 'aiohttp', 'google-auth'],
    platforms='any',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import aiohttp
from .authenticator import PyFCMAuthenticator
from .message import Message
from .errors import (
    UnspecifiedError, InvalidArgumentError, UnregisteredError,
    SenderIdMismatchError, QuotaExceededError, UnavailableError,
    InternalServerError, ThirdPartyAuthError
)


class AioPyFCM:

    def __init__(
        self,
        authenticator: PyFCMAuthenticator
    ):
        """AioPyFCM Initialization"""
        self.authenticator = authenticator
        self.session = None
        self.endpoint = "https://fcm.googleapis.com"

    async def __aenter__(self):
        """Open a session when entering the context."""
        self.session = aiohttp.ClientSession()
        return self

    async def __aexit__(self, exc_type, exc, tb):
        """Close the session when exiting the context."""
        await self.session.close()

    async def send(self, message: Message):
        """
        Send message to FCM server

        :param message: The message to send.
        :return: JSON response from the FCM Server.
        """
        if not self.session:
            # stateless mode: open a new session every time
            async with aiohttp.ClientSession() as session:
                return await self._post_message(message, session)
        else:
            # stateful mode: use the existing session
            return await self._post_message(message, self.session)

    async def _post_message(
        self,
        message: Message,
        session: aiohttp.ClientSession
    ):
        """
        Internal method to post a message.

        :param message: The message to send.
        :param session: The aiohttp session to use.
        """
        async with session.post(
            url=f"{self.endpoint}/v1/projects/{self.authenticator.project_id}/messages:send",
            headers={
                "Authorization": f"Bearer {self.authenticator.access_token}",
                "Content-Type": "application/json"
            },
            json={"message": message}
        ) as response:
            result = await response.json()
            return await self._handle_response(response, result)

    @staticmethod
    async def _handle_response(
        response: aiohttp.ClientResponse,
        result: dict
    ):
        """Handle various FCM response codes."""
        if response.status == 200:
            return result
        errors = {
            400: InvalidArgumentError,
            401: ThirdPartyAuthError,
            403: SenderIdMismatchError,
            404: UnregisteredError,
            429: QuotaExceededError,
            500: InternalServerError,
            503: UnavailableError,
        }
        raise errors.get(response.status, UnspecifiedError)(result)



"""
This module contains the ErrorCode for aiopyfcm.
https://firebase.google.com/docs/reference/fcm/rest/v1/ErrorCode
"""


class InvalidCredentialsError(Exception):
    """
    Raised when the credentials provided are invalid.
    """
    pass


class AioPyFCMError(Exception):
    """
    Raised when an error occurs in AioPyFCM.
    """
    pass


class UnspecifiedError(AioPyFCMError):
    """
    No more information is available about this error.
    """
    pass


class InvalidArgumentError(AioPyFCMError):
    """
    Request parameters were invalid.
    An extension of type google.rpc.BadRequest is returned to specify which field was invalid.
    """
    pass


class UnregisteredError(AioPyFCMError):
    """
    App instance was unregistered from FCM.
    This usually means that the token used is no longer valid and a new one must be used.
    """
    pass


class SenderIdMismatchError(AioPyFCMError):
    """
    The authenticated sender ID is different from the sender ID for the registration token.
    """
    pass


class QuotaExceededError(AioPyFCMError):
    """
    Sending limit exceeded for the message target.
    An extension of type google.rpc.QuotaFailure is returned to specify which quota was exceeded.
    """
    pass


class UnavailableError(AioPyFCMError):
    """The server is overloaded."""
    pass


class InternalServerError(AioPyFCMError):
    """An unknown internal error occurred."""
    pass


class ThirdPartyAuthError(AioPyFCMError):
    """APNs certificate or web push auth key was invalid or missing."""
    pass
